import { NextFunction, Request, Response } from 'express';
import { CreateAgencyDto } from '../dtos/agency.dto';
import { Agency } from '../interfaces/agency.interface';
import AgencyService from '../services/agency.service';
import { StatusCodes } from 'http-status-codes';
import { IResponse } from '../interfaces/response.interface';

class AgencyController {
  public agencyService = new AgencyService();
  public iResponse: IResponse;

  public getAgencies = async (req: Request, res: Response, next: NextFunction) => {
    try {
      const findAllAgenciesData: Agency[] = await this.agencyService.findAllAgencies();
      this.iResponse = { data: findAllAgenciesData, message: 'find all' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getAgencyById = async (req: Request, res: Response, next: NextFunction) => {
    const agencyId: string = req.params.id;

    
    try {
      const findOneAgencyData: Agency = await this.agencyService.findAgencyById(agencyId);
      this.iResponse = { data: findOneAgencyData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };
 
  public createAgency = async (req: Request, res: Response, next: NextFunction) => {
    const agencyData: CreateAgencyDto = req.body;
    try {
      const createAgencyData: Agency = await this.agencyService.createAgency(agencyData);
      this.iResponse = { data: createAgencyData, message: 'created' };
      res.status(StatusCodes.CREATED).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public updateAgency = async (req: Request, res: Response, next: NextFunction) => {
    const agencyId: string = req.params.id;
    const agencyData: Agency = req.body;

    try {
      const updateAgencyData: Agency = await this.agencyService.updateAgency(agencyId, agencyData);
      this.iResponse = { data: updateAgencyData, message: 'updated' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public deleteAgency = async (req: Request, res: Response, next: NextFunction) => {
    const agencyId: string = req.params.id;

    try {
      const deleteAgencyData: Agency = await this.agencyService.deleteAgencyData(agencyId);
      this.iResponse = { data: deleteAgencyData, message: 'deleted' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };
}

export default AgencyController;
