import { NextFunction, Request, Response } from 'express';
import { CreateApplicationDto } from '../dtos/application.dto';
import { Application } from '../interfaces/application.interface';
import applicationService from '../services/application.service';
import { StatusCodes } from 'http-status-codes';
import { IResponse } from '../interfaces/response.interface';

class ApplicationController {
  public applicationService = new applicationService();
  public iResponse: IResponse;

  //Create Idea
  public createIdea = async (req, res, next: NextFunction) => {
    const ideaData = req.body;
    console.log('inside controller', req.body)
    try {
      console.log('req.files',req.files)
      //Updating resources with s3 location
      if(req.files.proof){
      ideaData.proof = req.files.proof.map(file => {
        return { media: file.originalname};
      });
    }
      if(req.files.cv){
      ideaData.cv = req.files.cv.map(file => {
        return { media: file.originalname};
      });
      }
      if(req.files.award){
      ideaData.award = req.files.award.map(file => {
        return { media: file.originalname};
      });
      }
      if(req.files.benefit){
        ideaData.benefit = req.files.benefit.map(file => {
          return { media: file.originalname};
        });
      }


      ideaData.slug = ideaData.category.toString().split(' ').join('-');
      // res.status(StatusCodes.CREATED).json(this.iResponse);
      const createideaData: Application = await this.applicationService.createIdea(ideaData);
      this.iResponse = { data: createideaData, message: 'created' };
      res.status(StatusCodes.CREATED).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  //Update Idea
  public updateIdea = async (req, res, next: NextFunction) => {

    // console.log(req);
    
    const ideaId: string = req.body.id;
    const ideaData = req.body;

    console.log('iid',ideaData);
    console.log(ideaId);
    
    console.log('req.files',req.files)
    //Updating resources with s3 location
    if(req.files.proof){
    ideaData.proof = req.files.proof.map(file => {
      return { media: file.originalname};
    });
  }
    if(req.files.cv){
    ideaData.cv = req.files.cv.map(file => {
      return { media: file.originalname};
    });
    }
    if(req.files.award){
    ideaData.award = req.files.award.map(file => {
      return { media: file.originalname};
    });
    }
    if(req.files.benefit){
      ideaData.benefit = req.files.benefit.map(file => {
        return { media: file.originalname};
      });
    }
    
    try {
      const updateIdeaData: Application = await this.applicationService.updateIdea(ideaId, ideaData);
      this.iResponse = { data: updateIdeaData, message: 'updated' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  //Get Idea by ID
  public getIdeaById = async (req: Request, res: Response, next: NextFunction) => {
    const ideaId: string = req.params.id;
    try {
      const findOneApplicationData: Application = await this.applicationService.getIdeaById(ideaId);
      this.iResponse = { data: findOneApplicationData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };


  //Get Challenge Specific Ideas
  public getApplicationByChallengeId = async (req: Request, res: Response, next: NextFunction) => {
    const challengeId: string = req.params.id;
    try {
      const findOneApplicationData: Application = await this.applicationService.getIdeaByChallengeId(challengeId);
      this.iResponse = { data: findOneApplicationData, message: 'find' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  //Get User Specific Idea
  public getApplicationByUser = async (req: Request, res: Response, next: NextFunction) => {
    const userId: string = req.params.id;
    try {
      const findOneApplicationData: Application = await this.applicationService.findApplicationByUser(userId);
      this.iResponse = { data: findOneApplicationData, message: 'findAllIdeaByUser' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };


  

  //Old
  public getApplications = async (req: Request, res: Response, next: NextFunction) => {
    try {
      const findAllApplicationsData: Application[] = await this.applicationService.findAllApplication();
      this.iResponse = { data: findAllApplicationsData, message: 'find all' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getApplicationById = async (req: Request, res: Response, next: NextFunction) => {
    const applicationId: string = req.params.id;
    try {
      const findOneApplicationData: Application = await this.applicationService.findApplicationById(applicationId);
      this.iResponse = { data: findOneApplicationData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public updateApplication = async (req: Request, res: Response, next: NextFunction) => {
    const applicationId: string = req.params.id;
    const applicationData: Application = req.body;
    try {
      const updateApplicationData: Application = await this.applicationService.updateApplication(applicationId, {data: applicationData});
      this.iResponse = { data: updateApplicationData, message: 'updated' };
      console.log()
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public deleteApplication = async (req: Request, res: Response, next: NextFunction) => {
    const applicationId: string = req.params.id;

    try {
      const deleteApplicationData: Application = await this.applicationService.deleteApplicationData(applicationId);
      this.iResponse = { data: deleteApplicationData, message: 'deleted' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public createApplication = async (req: Request, res: Response, next: NextFunction) => {
    const applicationData: CreateApplicationDto = req.body;
    try {
      const createApplicationData: Application = await this.applicationService.createApplication(applicationData);
      this.iResponse = { data: createApplicationData, message: 'created' };
      res.status(StatusCodes.CREATED).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };
}

export default ApplicationController;
