import { NextFunction, Request, Response } from 'express';
import { CreateChallengeDto } from '../dtos/challenge.dto';
import { Challenge } from '../interfaces/challenge.interface';
import challengeService from '../services/challenge.service';
import { StatusCodes } from 'http-status-codes';
import { IResponse } from '../interfaces/response.interface';
import stepModel from '../models/step.model';
import agencyModel from '../models/agency.model';
import { isEmpty } from '../utils/util';
import { createUniqueId, generate_unique_id, uniqueIdIncrement } from '../utils/unique_id_generator';

class ChallengeController {
  public challengeService = new challengeService();
  public step = stepModel;
  public agency = agencyModel;
  public iResponse: IResponse;

  public getChallenges = async (req: Request, res: Response, next: NextFunction) => {
    try {
      const findAllChallengesData: Challenge[] = await this.challengeService.findAllChallenge();
      this.iResponse = { data: findAllChallengesData, message: 'find all' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getChallengeById = async (req: Request, res: Response, next: NextFunction) => {
    const challengeId: string = req.params.id;
    try {
      const findOneChallengeData: Challenge = await this.challengeService.findChallengeById(challengeId);
      this.iResponse = { data: findOneChallengeData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getChallengeStageById = async (req: Request, res: Response, next: NextFunction) => {
    const challengeId: string = req.params.id;
    try {
      const findOneChallengeData: any = await this.challengeService.getChallengeStageById(challengeId);
      this.iResponse = { data: findOneChallengeData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getChallengeByAgency = async (req: Request, res: Response, next: NextFunction) => {
    const challengeId: string = req.params.id;
    try {
      const findOneChallengeData: Challenge = await this.challengeService.findChallengeByAgency(challengeId);
      this.iResponse = { data: findOneChallengeData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public createChallenge = async (req, res, next: NextFunction) => {
    let challengeData = req.body;
    //Generating Challenge id
    let agency:any = await this.agency.findById(challengeData.agency_id);
    console.log(agency.agencyCode);
    
    let challenge_id = await generate_unique_id('challenge', agency.agencyCode);
    challengeData.challenge_id = challenge_id;
    // console.log(challengeData);

    try {
      let resource_data = req.body.resource_data? JSON.parse(req.body.resource_data) :'';
      //Updating resources with s3 location
      challengeData.resources = req.files.map(file => {
        return { media: file.location};
      });
      challengeData.logos = JSON.parse(challengeData.logos);
      challengeData.steps = JSON.parse(challengeData.steps);
      challengeData.slug = challengeData.category.toString().split(' ').join('-');
      const createChallengeData: Challenge = await this.challengeService.createChallenge(challengeData);
      if (createChallengeData) {
        uniqueIdIncrement('challenge', agency.agencyCode);
      }
      this.iResponse = { data: createChallengeData, message: 'created' };
      res.status(StatusCodes.CREATED).json(this.iResponse);
    } catch (error) {
      console.log(error);
      
      next(error);
    }
  };

  public updateChallenge = async (req, res, next: NextFunction) => {

    console.log(req);
    
    const challengeId: string = req.body.id;
    const challengeData = req.body;

    console.log(challengeData);
    console.log(challengeId);
    
    
    //Generating Challenge id
    // let agency:any = await this.agency.findById(challengeData.agency_id);
    // console.log(agency.agencyCode);
    
    // let challenge_id = await generate_unique_id('challenge', agency.agencyCode);
    // challengeData.challenge_id = challenge_id;
    // console.log(challengeData);

      // let resource_data = JSON.parse(req.body.resource_data);
      //Updating resources with s3 location
      challengeData.resources = ""
    challengeData.logos = JSON.parse(challengeData.logos)
      challengeData.steps = JSON.parse(challengeData.steps);
      challengeData.slug = challengeData.category.toString().split(' ').join('-');
    try {
      const updateChallengeData: Challenge = await this.challengeService.updateChallenge(challengeId, challengeData);
      this.iResponse = { data: updateChallengeData, message: 'updated' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public deleteChallenge = async (req: Request, res: Response, next: NextFunction) => {
    const challengeId: string = req.params.id;
    try {
      const deleteChallengeData: Challenge = await this.challengeService.deleteChallengeData(challengeId);
      this.iResponse = { data: deleteChallengeData, message: 'deleted' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };
}

export default ChallengeController;
