import { NextFunction, Request, Response } from 'express';
import { CreateRoleDto } from '../dtos/role.dto';
import { Role } from '../interfaces/role.interface';
import roleService from '../services/role.service';
import { StatusCodes } from 'http-status-codes';
import { IResponse } from '../interfaces/response.interface';

class RoleController {
  public roleService = new roleService();
  public iResponse: IResponse;

  public getRoles = async (req: Request, res: Response, next: NextFunction) => {
    try {
      const findAllRolesData: Role[] = await this.roleService.findAllRole();
      this.iResponse = { data: findAllRolesData, message: 'find all' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getRoleById = async (req: Request, res: Response, next: NextFunction) => {
    const roleId: string = req.params.id;

    try {
      const findOneRoleData: Role = await this.roleService.findRoleById(roleId);
      this.iResponse = { data: findOneRoleData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public createRole = async (req: Request, res: Response, next: NextFunction) => {
    const roleData: CreateRoleDto = req.body;

    try {
      const createRoleData: Role = await this.roleService.createRole(roleData);
      this.iResponse = { data: createRoleData, message: 'created' };
      res.status(StatusCodes.CREATED).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public updateRole = async (req: Request, res: Response, next: NextFunction) => {
    const roleId: string = req.params.id;
    const roleData: Role = req.body;

    try {
      const updateRoleData: Role = await this.roleService.updateRole(roleId, roleData);
      this.iResponse = { data: updateRoleData, message: 'updated' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public deleteRole = async (req: Request, res: Response, next: NextFunction) => {
    const roleId: string = req.params.id;

    try {
      const deleteRoleData: Role = await this.roleService.deleteRoleData(roleId);
      this.iResponse = { data: deleteRoleData, message: 'deleted' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };
}

export default RoleController;
