import { NextFunction, Request, Response } from 'express';
import { CreateStepDto } from '../dtos/step.dto';
import { Step } from '../interfaces/step.interface';
import stepService from '../services/step.service';
import { StatusCodes } from 'http-status-codes';
import { IResponse } from '../interfaces/response.interface';

class StepController {
  public stepService = new stepService();
  public iResponse: IResponse;

  public getSteps = async (req: Request, res: Response, next: NextFunction) => {
    try {
      const findAllStepsData: Step[] = await this.stepService.findAllStep();
      this.iResponse = { data: findAllStepsData, message: 'find all' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getStepById = async (req: Request, res: Response, next: NextFunction) => {
    const stepId: string = req.params.id;
    try {
      const findOneStepData: Step = await this.stepService.findStepById(stepId);
      this.iResponse = { data: findOneStepData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public createStep = async (req: Request, res: Response, next: NextFunction) => {
    const stepData: CreateStepDto = req.body;
    try {
      const createStepData: Step = await this.stepService.createStep(stepData);
      this.iResponse = { data: createStepData, message: 'created' };
      res.status(StatusCodes.CREATED).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public updateStep = async (req: Request, res: Response, next: NextFunction) => {
    const stepId: string = req.params.id;
    const stepData: Step = req.body;
    try {
      const updateStepData: Step = await this.stepService.updateStep(stepId, stepData);
      this.iResponse = { data: updateStepData, message: 'updated' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public deleteStep = async (req: Request, res: Response, next: NextFunction) => {
    const stepId: string = req.params.id;

    try {
      const deleteStepData: Step = await this.stepService.deleteStepData(stepId);
      this.iResponse = { data: deleteStepData, message: 'deleted' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };
}

export default StepController;
