import { IResponse } from '../interfaces/response.interface';
import { NextFunction, Request, Response } from 'express';
import { CreateUserDto } from '../dtos/user.dto';
import { User } from '../interfaces/user.interface';
import userService from '../services/user.service';
import { StatusCodes } from 'http-status-codes';
class UserController {
  public userService = new userService();
  public iResponse: IResponse;

  public getUsers = async (req: Request, res: Response, next: NextFunction) => {
    try {
      const findAllUsersData: User[] = await this.userService.findAllUser();
      this.iResponse = { data: findAllUsersData, message: 'find all' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getUserById = async (req: Request, res: Response, next: NextFunction) => {
    const userId: string = req.params.id;

    try {
      const findOneUserData: User = await this.userService.findUserById(userId);
      this.iResponse = { data: findOneUserData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public createUser = async (req: Request, res: Response, next: NextFunction) => {
    const userData: CreateUserDto = req.body;

    try {
      const createUserData: User = await this.userService.createUser(userData);
      this.iResponse = { data: createUserData, message: 'created' };
      res.status(StatusCodes.CREATED).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public updateUser = async (req: Request, res: Response, next: NextFunction) => {
    const userId: string = req.params.id;
    const userData: User = req.body;

    try {
      const updateUserData: User = await this.userService.updateUser(userId, userData);
      this.iResponse = { data: updateUserData, message: 'updated' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public deleteUser = async (req: Request, res: Response, next: NextFunction) => {
    const userId: string = req.params.id;

    try {
      const deleteUserData: User = await this.userService.deleteUserData(userId);
      this.iResponse = { data: deleteUserData, message: 'deleted' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };
}

export default UserController;
