import { NextFunction, Request, Response } from 'express';
import { CreateVideoDto } from '../dtos/video.dto';
import { Video } from '../interfaces/video.interface';
import videoService from '../services/video.service';
import { StatusCodes } from 'http-status-codes';
import { IResponse } from '../interfaces/response.interface';

class VideoController {
  public videoService = new videoService();
  public iResponse: IResponse;

  public getVideos = async (req: Request, res: Response, next: NextFunction) => {
    try {
      const findAllVideosData: Video[] = await this.videoService.findAllVideo();
      this.iResponse = { data: findAllVideosData, message: 'find all' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public getVideoById = async (req: Request, res: Response, next: NextFunction) => {
    const videoId: string = req.params.id;
    try {
      const findOneVideoData: Video = await this.videoService.findVideoById(videoId);
      this.iResponse = { data: findOneVideoData, message: 'findOne' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public createVideo = async (req: Request, res: Response, next: NextFunction) => {
    const videoData: CreateVideoDto = req.body;
    try {
      const createVideoData: Video = await this.videoService.createVideo(videoData);
      this.iResponse = { data: createVideoData, message: 'created' };
      res.status(StatusCodes.CREATED).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public updateVideo = async (req: Request, res: Response, next: NextFunction) => {
    const videoId: string = req.params.id;
    const videoData: Video = req.body;
    try {
      const updateVideoData: Video = await this.videoService.updateVideo(videoId, videoData);
      this.iResponse = { data: updateVideoData, message: 'updated' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };

  public deleteVideo = async (req: Request, res: Response, next: NextFunction) => {
    const videoId: string = req.params.id;

    try {
      const deleteVideoData: Video = await this.videoService.deleteVideoData(videoId);
      this.iResponse = { data: deleteVideoData, message: 'deleted' };
      res.status(StatusCodes.OK).json(this.iResponse);
    } catch (error) {
      next(error);
    }
  };
}

export default VideoController;
