import { NextFunction, Response } from 'express';
import jwt from 'jsonwebtoken';
import HttpException from '../exceptions/HttpException';
import { DataStoredInToken, RequestWithUser } from '../interfaces/auth.interface';
import userModel from '../models/user.model';

const authMiddleware = async (req: RequestWithUser, res: Response, next: NextFunction) => {
  try {
    const token = req.body.token || req.query.token || req.headers["authorization"];
    if (token) {
      const secret = process.env.JWT_SECRET;
      const verificationResponse = (await jwt.verify(token.authorization, secret)) as DataStoredInToken;
      console.log(verificationResponse);
      const userId = verificationResponse._id;
      const findUser = await userModel.findById(userId);

      if (findUser) {
        req.user = findUser;
        next();
      } else {
        next(new HttpException(401, 'Invalid Authentication Token'));
      }
    } else {
      next(new HttpException(401, 'Authentication Token Missing'));
    }
  } catch (error) {
    next(new HttpException(401, 'Invalid Authentication Token'));
  }
};

export default authMiddleware;
