import { model, Schema, Document, SchemaTypes } from 'mongoose';
import { Agency } from '../interfaces/agency.interface';
import paginate from './plugins/paginate.plugin';
import toJSON from './plugins/toJSON.plugin';

const agencySchema: Schema = new Schema(
  {
    agencyName: {
      type: String,
      required: true,
      unique: true,
      lowercase: true,
    },
    agencyCode: {
      type: String,
      required: true,
      unique: true,
      lowercase: true,
    },
    email: {
        type: String,
        required: true,
        unique: true,
        lowercase: true,
    },
    userId: {
        type: SchemaTypes.ObjectId,
        ref: 'User',
        required: true,
      },
    status: {
      type: String,
      enum: ['ACTIVE', 'INACTIVE'],
      required: true,
    },
  },
  {
    timestamps: true,
  },
);

agencySchema.plugin(toJSON);
agencySchema.plugin(paginate);

const agencyModel = model<Agency & Document>('Agency', agencySchema);

export default agencyModel;
