import { model, Schema, Document, SchemaTypes } from 'mongoose';
import mongoose from 'mongoose';
import { Challenge } from '../interfaces/challenge.interface';
import paginate from './plugins/paginate.plugin';
import toJSON from './plugins/toJSON.plugin';

const challengeSchema: Schema = new Schema(
  {
    agency_id: {
      type: SchemaTypes.ObjectId,
      ref: 'Agency',
      required: true,
    },
    challenge_id: {
      type: String,
    },
    category: {
      type: String,
      required: true,
    },
    year:{
      type: String,
      required: true,
    },
    sdg:{
      type: SchemaTypes.ObjectId,
      ref: 'sdg',
      required: true,
    },
    slug: {
      type: String,
      required: false,
    },
    description: {
      type: String,
      required: true,
    },
    key_areas: {
      type: String,
    },
    who_can_apply: {
      type: String,
    },
    incentive: {
      type: String,
    },
    parameter_for_selection: {
      type: String,
    },
    guideline: {
      type: String,
    },
    header_line: {
      type: String,
    },
    web_link: {
      type: String,
    },
    c_image: {
      type: String,
    },
    logos: [],
    resources: [],
    twitter_link: {
      type: String,
    },
    linkedin_link: {
      type: String,
    },
    facebook_link: {
      type: String,
    },
    steps: [],
    status: {
      type: String,
      default: 'ACTIVE',
      enum: ['ACTIVE', 'INACTIVE'],
      required: true,
    },
  },
  {
    timestamps: true,
  },
);

challengeSchema.plugin(toJSON);
challengeSchema.plugin(paginate);

const challengeModel = model<Challenge & Document>('Challenge', challengeSchema);
export default challengeModel;
