import { model, Schema, Document } from 'mongoose';
import { Role } from '../interfaces/role.interface';
import paginate from './plugins/paginate.plugin';
import toJSON from './plugins/toJSON.plugin';

const roleSchema: Schema = new Schema(
  {
    name: {
      type: String,
      required: true,
      unique: true,
      lowercase: true,
    },
    permissions: {
      type: Object,
    },
    status: {
      type: String,
      enum: ['ACTIVE', 'INACTIVE'],
      required: true,
    },
  },
  {
    timestamps: true,
  },
);

roleSchema.plugin(toJSON);
roleSchema.plugin(paginate);

const roleModel = model<Role & Document>('Role', roleSchema);

export default roleModel;
