import { model, Schema, Document, SchemaTypes } from 'mongoose';
import { User } from '../interfaces/user.interface';
import paginate from './plugins/paginate.plugin';
import toJSON from './plugins/toJSON.plugin';

const userSchema: Schema = new Schema(
  {
    email: {
      type: String,
      required: true,
      unique: true,
      lowercase: true,
    },
    password: {
      type: String,
      required: true,
      trim: true,
      minlength: 8,
      validate(value) {
        if (!value.match(/\d/) || !value.match(/[a-zA-Z]/)) {
          throw new Error('Password must contain at least one letter and one number');
        }
      },
      private: true, // used by the toJSON plugin
    },
    first_name: {
      type: String,
      required: true,
      trim: true,
    },
    last_name: {
      type: String,
      required: true,
      trim: true,
    },
    nationality: {
      type: String,
      trim: true,
    },
    country: {
      type: String,
      trim:true
    },
    age: {
      type: String,
    },
    gender: {
      type: String,
    },
    profession: {
      type: String,
    },
    state: {
      type: String,
    },
    city: {
      type: String,
    },
    address: {
      type: String,
    },
    phone: {
      type: String,
      required: true,
    },
    pincode: {
      type: String,
      default: '',
    },
    role: {
      type: SchemaTypes.ObjectId,
      ref: 'Role',
      required: true,
    },
    status: {
      type: String,
      enum: ['ACTIVE', 'INACTIVE'],
      default: 'ACTIVE',
      required: true,
    },
  },
  {
    timestamps: true,
  },
);

userSchema.plugin(toJSON);
userSchema.plugin(paginate);

const userModel = model<User & Document>('User', userSchema);

export default userModel;
