import { model, Schema, Document } from 'mongoose';
import { Video } from '../interfaces/video.interface';
import paginate from './plugins/paginate.plugin';
import toJSON from './plugins/toJSON.plugin';

const videoSchema: Schema = new Schema(
  {
    title: {
      type: String,
      required: true,
    },
    link: {
      type: String,
      required: true,
    },
    sort_order: {
      type: Number,
    },
    status: {
      type: String,
      enum: ['ACTIVE', 'INACTIVE'],
      default: 'ACTIVE',
      required: true,
    },
  },
  {
    timestamps: true,
  },
);

videoSchema.plugin(toJSON);
videoSchema.plugin(paginate);

const videoModel = model<Video & Document>('Video', videoSchema);

export default videoModel;
