import { Router } from 'express';
import AgencyController from '../controllers/agency.controller';
import { CreateAgencyDto } from '../dtos/agency.dto';
import Route from '../interfaces/route.interface';
import validationMiddleware from '../middlewares/validation.middleware';
import authMiddleware from '../middlewares/auth.middleware';
class AgencyRoute implements Route {
  public path = '/agency';
  public router = Router();
  public agencyController = new AgencyController();

  constructor() {
    this.initializeRoutes();
  }

  private initializeRoutes() {
    this.router.get(`${this.path}`,  this.agencyController.getAgencies);
    this.router.get(`${this.path}/:id`, authMiddleware, this.agencyController.getAgencyById);
    this.router.post(`${this.path}`, authMiddleware, validationMiddleware(CreateAgencyDto, 'body'), this.agencyController.createAgency);
    this.router.put(`${this.path}/:id`, authMiddleware, validationMiddleware(CreateAgencyDto, 'body', true), this.agencyController.updateAgency);
    this.router.delete(`${this.path}/:id`, authMiddleware, this.agencyController.deleteAgency);
  }
}

export default AgencyRoute;
