import { Router } from 'express';
import ApplicationController from '../controllers/application.controller';
import { CreateApplicationDto } from '../dtos/application.dto';
import Route from '../interfaces/route.interface';
import validationMiddleware from '../middlewares/validation.middleware';
import authMiddleware from '../middlewares/auth.middleware';
import upload from '../utils/upload';

class ApplicationRoute implements Route {
  public path_old = '/applications';
  public path = '/idea'
  public router = Router();

  public applicationController = new ApplicationController();
  

  constructor() {
    this.initializeRoutes();
  }

  private initializeRoutes() {
    this.router.get(`${this.path}`, this.applicationController.getApplications);
    this.router.get('/idea', this.applicationController.getApplications);
    this.router.get(`${this.path}/byUser/:id`, this.applicationController.getApplicationByUser);
    // this.router.get(`${this.path}/:id`, this.applicationController.getApplicationById);
    this.router.get(`${this.path}/challenge/:id`, this.applicationController.getApplicationByChallengeId);
    this.router.post(
      `${this.path}`,
      authMiddleware,
      validationMiddleware(CreateApplicationDto, 'body'),
      this.applicationController.createApplication,
    );
    
    this.router.post(`${this.path}/create`, upload.fields([{name: 'cv', maxCount: 1},{name: 'proof', maxCount: 1},{name: 'award', maxCount: 1},{name: 'benefit', maxCount: 1}]) ,this.applicationController.createIdea);
    this.router.get(`${this.path}/:id`, this.applicationController.getIdeaById);
    this.router.put(
      `${this.path}/:id`,
      authMiddleware,
      validationMiddleware(CreateApplicationDto, 'body', true),
      this.applicationController.updateApplication,
    );
    this.router.post(
      `${this.path}/update`,
      upload.fields([{name: 'cv', maxCount: 1},{name: 'proof', maxCount: 1},{name: 'award', maxCount: 1},{name: 'benefit', maxCount: 1}]),
      this.applicationController.updateIdea,
    );
    this.router.delete(`${this.path}/:id`, authMiddleware, this.applicationController.deleteApplication);
  }
}

export default ApplicationRoute;
