import { Router } from 'express';
import ChallengeController from '../controllers/challenge.controller';
import { CreateChallengeDto } from '../dtos/challenge.dto';
import Route from '../interfaces/route.interface';
import validationMiddleware from '../middlewares/validation.middleware';
import authMiddleware from '../middlewares/auth.middleware';
import upload from '../utils/upload';
class ChallengeRoute implements Route {
  public path = '/challenges';
  public router = Router();
  public challengeController = new ChallengeController();

  constructor() {
    this.initializeRoutes();
  }

  private initializeRoutes() {
    this.router.get(`${this.path}`,this.challengeController.getChallenges);
    this.router.get(`${this.path}/:id`, this.challengeController.getChallengeById);
    this.router.get(`${this.path}/stage/:id`, this.challengeController.getChallengeStageById);
    this.router.get(`${this.path}/agency/:id`, this.challengeController.getChallengeByAgency);
    this.router.post(`${this.path}/create`,  upload.array('resources',10),this.challengeController.createChallenge);
    this.router.post(
      `${this.path}/update`,upload.array('resources',10),
      this.challengeController.updateChallenge,
    );
    this.router.delete(`${this.path}/:id`, authMiddleware, this.challengeController.deleteChallenge);
  }
}

export default ChallengeRoute;
