import { Router } from 'express';
import RoleController from '../controllers/role.controller';
import { CreateRoleDto } from '../dtos/role.dto';
import Route from '../interfaces/route.interface';
import validationMiddleware from '../middlewares/validation.middleware';
import authMiddleware from '../middlewares/auth.middleware';
class RoleRoute implements Route {
  public path = '/roles';
  public router = Router();
  public roleController = new RoleController();

  constructor() {
    this.initializeRoutes();
  }

  private initializeRoutes() {
    this.router.get(`${this.path}`, this.roleController.getRoles);
    this.router.get(`${this.path}/:id`, authMiddleware, this.roleController.getRoleById);
    this.router.post(`${this.path}`, authMiddleware, validationMiddleware(CreateRoleDto, 'body'), this.roleController.createRole);
    this.router.put(`${this.path}/:id`, authMiddleware, validationMiddleware(CreateRoleDto, 'body', true), this.roleController.updateRole);
    this.router.delete(`${this.path}/:id`, authMiddleware, this.roleController.deleteRole);
  }
}

export default RoleRoute;
