import { Router } from 'express';
import StepController from '../controllers/step.controller';
import { CreateStepDto } from '../dtos/step.dto';
import Route from '../interfaces/route.interface';
import validationMiddleware from '../middlewares/validation.middleware';
import authMiddleware from '../middlewares/auth.middleware';
class StepRoute implements Route {
  public path = '/steps';
  public router = Router();
  public stepController = new StepController();

  constructor() {
    this.initializeRoutes();
  }

  private initializeRoutes() {
    this.router.get(`${this.path}`, authMiddleware, this.stepController.getSteps);
    this.router.get(`${this.path}/:id`, authMiddleware, this.stepController.getStepById);
    this.router.post(`${this.path}`, authMiddleware, validationMiddleware(CreateStepDto, 'body'), this.stepController.createStep);
    this.router.put(`${this.path}/:id`, authMiddleware, validationMiddleware(CreateStepDto, 'body', true), this.stepController.updateStep);
    this.router.delete(`${this.path}/:id`, authMiddleware, this.stepController.deleteStep);
  }
}

export default StepRoute;
