import { Router } from 'express';
import VideoController from '../controllers/video.controller';
import { CreateVideoDto } from '../dtos/video.dto';
import Route from '../interfaces/route.interface';
import validationMiddleware from '../middlewares/validation.middleware';
import authMiddleware from '../middlewares/auth.middleware';

class VideoRoute implements Route {
  public path = '/videos';
  public router = Router();
  public videoController = new VideoController();

  constructor() {
    this.initializeRoutes();
  }

  private initializeRoutes() {
    this.router.get(`${this.path}`, this.videoController.getVideos);
    this.router.get(`${this.path}/:id`, this.videoController.getVideoById);
    this.router.post(`${this.path}`, authMiddleware, validationMiddleware(CreateVideoDto, 'body'), this.videoController.createVideo);
    this.router.put(`${this.path}/:id`, authMiddleware, validationMiddleware(CreateVideoDto, 'body', true), this.videoController.updateVideo);
    this.router.delete(`${this.path}/:id`, authMiddleware, this.videoController.deleteVideo);
  }
}

export default VideoRoute;
