import { CreateAgencyDto } from '../dtos/agency.dto';
import HttpException from '../exceptions/HttpException';
import { Agency } from '../interfaces/agency.interface';
import agencyModel from '../models/agency.model';
import { isEmpty } from '../utils/util';
import {sendMail} from '../utils/mail';
import userModel from '../models/user.model';
import uniqueidModal from '../models/uniqueid.model';
import { User } from '../interfaces/user.interface';
import { createUniqueId } from '../utils/unique_id_generator';

class AgencyService {
  public agencies = agencyModel;
  public users = userModel;

  public async findAllAgencies(): Promise<Agency[]> {
    const agencies: Agency[] = await this.agencies.find();
    return agencies;
  }

  public async findAgencyById(agencyId: string): Promise<Agency> {
    const findAgency: Agency = await this.agencies.findById(agencyId);
    if (!findAgency) throw new HttpException(409, 'Agency Not Found');

    return findAgency;
  }

  public async createAgency(agencyData: CreateAgencyDto): Promise<Agency> {
    if (isEmpty(agencyData)) throw new HttpException(400, 'Agency Data Not Found');

    const findAgency: Agency = await this.agencies.findOne({ agencyName: agencyData.agencyName, email: agencyData.email });
    if (findAgency) throw new HttpException(409, `${agencyData.agencyName} already exists`);
    const createAgencyData: Agency = await this.agencies.create(agencyData);
    if(createAgencyData){
      let uniqueId = await createUniqueId("challenge",agencyData.agencyCode)
      if(uniqueId == "UID created"){
        console.log("sendgrid key ======>", process.env.SENDGRID_API_KEY);
        // const msg = {
        //   to: createAgencyData.email,
        //   from: 'rohit.singhal@un.org',
        //   subject: 'Agency Registered on GoChange',
        //   html: `<strong>Registered successfully on goChange!!!!</strong> <br> <strong>Login credentials are given below </strong> <br> <strong> Username: </strong> <p> ${createAgencyData.email} </p> <br> <strong> Password: </strong> <p> ${createAgencyData.agencyName}@123 </p>`,
        // };
        // sendMail(msg);
      }
    
    }
    return createAgencyData;
  }


  

  public async updateAgency(agencyId: string, agencyData: Agency): Promise<Agency> {
    if (isEmpty(agencyData)) throw new HttpException(400, 'Agency Data Not Found');
    const updateUserById: User = await this.users.findByIdAndUpdate(agencyData.userId, agencyData);
    const updateAgencyById: Agency = await this.agencies.findByIdAndUpdate(agencyId, agencyData);
    if (!updateAgencyById) throw new HttpException(409, 'Agency Data Not Found');

    return updateAgencyById;
  }

  public async deleteAgencyData(agencyId: string): Promise<Agency> {
    const deleteAgencyById: Agency = await this.agencies.findByIdAndDelete(agencyId);
    if (!deleteAgencyById) throw new HttpException(409, 'Agency Data Not Found');

    return deleteAgencyById;
  }
}

export default AgencyService;
