import { CreateApplicationDto } from '../dtos/application.dto';
import HttpException from '../exceptions/HttpException';
import { Application } from '../interfaces/application.interface';
import applicationModel from '../models/application.model';
import { isEmpty } from '../utils/util';

class ApplicationService {
  public applications = applicationModel;

  public async findAllApplication(): Promise<Application[]> {
    const applications: Application[] = await this.applications.find();
    return applications;
  }

  public async findApplicationById(applicationId: string): Promise<Application> {
    const findApplication: Application = await this.applications.findOne({ _id: applicationId });
    if (!findApplication) throw new HttpException(409, 'Application Not Found');

    return findApplication;
  }

  public async findApplicationByUser(userId: string): Promise<any> {
    const findApplication: any = await this.applications.find({ user: userId }).populate({path:'challenge',populate: {
      path: 'agency_id',
      model: 'Agency'
    } }).sort({updatedAt:-1});
    if (!findApplication) throw new HttpException(409, 'Application Not Found');
    return findApplication;
  }


  public async createApplication(applicationData: CreateApplicationDto): Promise<Application> {
    if (isEmpty(applicationData)) throw new HttpException(400, 'Application Data Not Found');

    const findApplication: Application = await this.applications.findOne({ user: applicationData.user, challenge: applicationData.challenge });
    if (findApplication) throw new HttpException(409, `Application already exists for this challenge`);
    const createApplicationData: Application = await this.applications.create(applicationData);
    return createApplicationData;
  }


  //Create Idea
  public async createIdea(ideaData): Promise<Application> {
    console.log('application service', ideaData)
    if (isEmpty(ideaData)) throw new HttpException(400, 'Application Data Not Found');
    const createApplicationData: Application = await this.applications.create({data: ideaData,challenge: ideaData.challenge_id, user: ideaData.user_id});
    return createApplicationData;
  }


  //Update Idea
  public async updateIdea(ideaId: string, ideaData: Application): Promise<Application> {
    if (isEmpty(ideaId)) throw new HttpException(400, 'Application  Not Found');
    const updateIdeaById: Application = await this.applications.findByIdAndUpdate(ideaId, {$set:{data:ideaData}});
    if (!updateIdeaById) throw new HttpException(409, 'Challenge Data Not Found');

    return updateIdeaById;
  }

  public async getIdeaById(ideaid: string): Promise<Application> {
    const findApplication: Application = await this.applications.findOne({ _id: ideaid }).populate('challenge');
    if (!findApplication) throw new HttpException(409, 'Application Not Found');

    return findApplication;
  }

  public async getIdeaByChallengeId(challengeid: string): Promise<any> {
    const findApplication = await this.applications.find({ challenge : challengeid });
    if (!findApplication) throw new HttpException(409, 'Application Not Found');

    return findApplication;
  }

  public async updateApplication(applicationId: string, applicationData: Application): Promise<Application> {
    if (isEmpty(applicationData)) throw new HttpException(400, 'Application Data Not Found');
    const updateApplicationById: Application = await this.applications.findByIdAndUpdate(applicationId, applicationData);
    if (!updateApplicationById) throw new HttpException(409, 'Application Data Not Found');

    return updateApplicationById;
  }

  public async deleteApplicationData(applicationId: string): Promise<Application> {
    const deleteApplicationById: Application = await this.applications.findByIdAndDelete(applicationId);
    if (!deleteApplicationById) throw new HttpException(409, 'Application Data Not Found');

    return deleteApplicationById;
  }
}

export default ApplicationService;
