import bcrypt from 'bcrypt';
import jwt from 'jsonwebtoken';
import { CreateUserDto } from '../dtos/user.dto';
import HttpException from '../exceptions/HttpException';
import { DataStoredInToken, TokenData } from '../interfaces/auth.interface';
import { User } from '../interfaces/user.interface';
import userModel from '../models/user.model';
import { isEmpty } from '../utils/util';

class AuthService {
  public users = userModel;

  public async signup(userData: CreateUserDto): Promise<User> {
    if (isEmpty(userData)) throw new HttpException(400, "You're not userData");
    const findUser: User = await this.users.findOne({ email: userData.email });
    if (findUser) throw new HttpException(409, `You're email ${userData.email} already exists`);
    const hashedPassword = await bcrypt.hash(userData.password, 10);
    const createUserData: User = await this.users.create({ ...userData, password: hashedPassword });
    return createUserData;
  }

  public async login(userData: CreateUserDto): Promise<any> {
    try {
  
    if (isEmpty(userData)) throw new HttpException(400, "You're not userData");
    let hashPwd = await bcrypt.hash(userData.password,10);
    console.log(hashPwd);
    
    // console.log();
    
    let findUser: any = await this.users.aggregate([
      {
        $match: {
          email: userData.email,
        },
      },
      {
        $lookup: {
          from: 'roles',
          localField: 'role',
          foreignField: '_id',
          as: 'role',
        },
      },
      {
        $unwind: {
          path: '$role',
          preserveNullAndEmptyArrays: true,
        },
      },
      {
        $lookup: {
          from: 'agencies',
          localField: '_id',
          foreignField: 'userId',
          as: 'agency',
        },
      },
      {
        $unwind: {
          path: '$agency',
          preserveNullAndEmptyArrays: true,
        },
      }
    ]);
    console.log(findUser);
    if (!findUser.length) {
      return {
        status: 409,
        error:true,
        message: `Your email ${userData.email} not found`,
        data: null
      };
    }else
    {
      findUser = findUser[0];
      console.log(userData.password);
  
      const isPasswordMatching: boolean = await bcrypt.compare(userData.password, findUser.password);
      console.log(isPasswordMatching);
      
      if (!isPasswordMatching)  return {
        status: 409,
        error: true,
        message: `Your password not matching`,
        data: null
      }; 
      const tokenData = this.createToken(findUser);
      findUser.token = tokenData.token;
      const cookie = this.createCookie(tokenData);
      return {
        status: 200,
        error: null,
        message:"Login Successfull",
        data: { cookie, findUser }
      }
    }  
  } catch (error) {
    console.log(error);
  }
  }

  public async logout(userData: User): Promise<User> {
    if (isEmpty(userData)) throw new HttpException(400, "You're not userData");
    const findUser: User = await this.users.findOne({ password: userData.password });
    if (!findUser) throw new HttpException(409, "You're not user");
    return findUser;
  }

  public createToken(user: User): TokenData {
    const dataStoredInToken: DataStoredInToken = { _id: user._id };
    const secret: string = process.env.JWT_SECRET;
    const expiresIn: number = 60 * 60;
    return { expiresIn, token: jwt.sign(dataStoredInToken, secret, { expiresIn }) };
  }

  public createCookie(tokenData: TokenData): string {
    return `Authorization=${tokenData.token}; HttpOnly; Max-Age=${tokenData.expiresIn};`;
  }
}

export default AuthService;
