import { CreateRoleDto } from '../dtos/role.dto';
import HttpException from '../exceptions/HttpException';
import { Role } from '../interfaces/role.interface';
import roleModel from '../models/role.model';
import { isEmpty } from '../utils/util';

class RoleService {
  public roles = roleModel;

  public async findAllRole(): Promise<Role[]> {
    const roles: Role[] = await this.roles.find();
    return roles;
  }

  public async findRoleById(roleId: string): Promise<Role> {
    const findRole: Role = await this.roles.findById(roleId);
    if (!findRole) throw new HttpException(409, 'Role Not Found');

    return findRole;
  }

  public async createRole(roleData: CreateRoleDto): Promise<Role> {
    if (isEmpty(roleData)) throw new HttpException(400, 'Role Data Not Found');

    const findRole: Role = await this.roles.findOne({ name: roleData.name });
    if (findRole) throw new HttpException(409, `${roleData.name} already exists`);
    const createRoleData: Role = await this.roles.create(roleData);
    return createRoleData;
  }

  public async updateRole(roleId: string, roleData: Role): Promise<Role> {
    if (isEmpty(roleData)) throw new HttpException(400, 'Role Data Not Found');
    const updateRoleById: Role = await this.roles.findByIdAndUpdate(roleId, roleData);
    if (!updateRoleById) throw new HttpException(409, 'Role Data Not Found');

    return updateRoleById;
  }

  public async deleteRoleData(roleId: string): Promise<Role> {
    const deleteRoleById: Role = await this.roles.findByIdAndDelete(roleId);
    if (!deleteRoleById) throw new HttpException(409, 'Role Data Not Found');

    return deleteRoleById;
  }
}

export default RoleService;
