import { CreateStepDto } from '../dtos/step.dto';
import HttpException from '../exceptions/HttpException';
import { Step } from '../interfaces/step.interface';
import stepModel from '../models/step.model';
import { isEmpty } from '../utils/util';

class StepService {
  public steps = stepModel;

  public async findAllStep(): Promise<Step[]> {
    const steps: Step[] = await this.steps.find();
    return steps;
  }

  public async findStepById(stepId: string): Promise<Step> {
    const findStep: Step = await this.steps.findById(stepId);
    if (!findStep) throw new HttpException(409, 'Step Not Found');

    return findStep;
  }

  public async createStep(stepData: CreateStepDto): Promise<Step> {
    if (isEmpty(stepData)) throw new HttpException(400, 'Step Data Not Found');

    const findStep: Step = await this.steps.findOne({ name: stepData.name });
    if (findStep) throw new HttpException(409, `${stepData.name} already exists`);
    const createStepData: Step = await this.steps.create(stepData);
    return createStepData;
  }

  public async updateStep(stepId: string, stepData: Step): Promise<Step> {
    if (isEmpty(stepData)) throw new HttpException(400, 'Step Data Not Found');
    const updateStepById: Step = await this.steps.findByIdAndUpdate(stepId, stepData);
    if (!updateStepById) throw new HttpException(409, 'Step Data Not Found');

    return updateStepById;
  }

  public async deleteStepData(stepId: string): Promise<Step> {
    const deleteStepById: Step = await this.steps.findByIdAndDelete(stepId);
    if (!deleteStepById) throw new HttpException(409, 'Step Data Not Found');

    return deleteStepById;
  }
}

export default StepService;
