import { CreateVideoDto } from '../dtos/video.dto';
import HttpException from '../exceptions/HttpException';
import { Video } from '../interfaces/video.interface';
import videoModel from '../models/video.model';
import { isEmpty } from '../utils/util';

class VideoService {
  public videos = videoModel;

  public async findAllVideo(): Promise<Video[]> {
    const videos: Video[] = await this.videos.find();
    return videos;
  }

  public async findVideoById(videoId: string): Promise<Video> {
    const findVideo: Video = await this.videos.findById(videoId);
    if (!findVideo) throw new HttpException(409, 'Video Not Found');

    return findVideo;
  }

  public async createVideo(videoData: CreateVideoDto): Promise<Video> {
    if (isEmpty(videoData)) throw new HttpException(400, 'Video Data Not Found');
    const createVideoData: Video = await this.videos.create(videoData);
    return createVideoData;
  }

  public async updateVideo(videoId: string, videoData: Video): Promise<Video> {
    if (isEmpty(videoData)) throw new HttpException(400, 'Video Data Not Found');
    const updateVideoById: Video = await this.videos.findByIdAndUpdate(videoId, videoData);
    if (!updateVideoById) throw new HttpException(409, 'Video Data Not Found');

    return updateVideoById;
  }

  public async deleteVideoData(videoId: string): Promise<Video> {
    const deleteVideoById: Video = await this.videos.findByIdAndDelete(videoId);
    if (!deleteVideoById) throw new HttpException(409, 'Video Data Not Found');

    return deleteVideoById;
  }
}

export default VideoService;
